//============================================================================
// Name: sincX_mult_precision.cpp
// Author: Saurav
// Copyright: May be distributed freely
// Description : sin(x) with 4 different levels of precision
//============================================================================

#include <iostream>
#include <stdio.h>
#include <iomanip>
#include <boost/multiprecision/cpp_dec_float.hpp>
#include <boost/math/constants/constants.hpp>
#include <limits>

using boost::multiprecision::cpp_dec_float_50;

template<typename T>
inline T sinc(T r)
{
	T temp = sin(r)/r;
	if (temp > std::numeric_limits<T>::epsilon())
		return temp;
	else
		return 1;
}


int main(int, char**){

	int ang_num, ang_denom;
	ang_num = 5;
	ang_denom = 100;// Don't set to zero. Provided to increase the precision of angle.
					// Use as ang_denom = 100, 1000, 100000, 1000000.. and so on.

	const float ang_f(static_cast<float>(ang_num) / ang_denom);
	const float sinc_f = sinc(ang_f);

	const double ang_d(static_cast<double>(ang_num) / ang_denom);
	const double sinc_d = sinc(ang_d);

	const long double ang_ld(static_cast<long double>(ang_num) / ang_denom);
	const long double sinc_ld = sinc(ang_ld);

	const cpp_dec_float_50 ang_mp(static_cast<cpp_dec_float_50>(ang_num) / ang_denom);
	const cpp_dec_float_50 sinc_mp = sinc(ang_mp);

	std::cout
		<< std::setprecision(std::numeric_limits<float>::digits10)
		<< sinc_f
		<< std::endl;

	std::cout
		<< std::setprecision(std::numeric_limits<double>::digits10)
		<< sinc_d
		<< std::endl;

	std::cout
		<< std::setprecision(std::numeric_limits<long double>::digits10)
		<< sinc_ld
		<< std::endl;

	std::cout
		<< std::setprecision(std::numeric_limits<cpp_dec_float_50>::digits10)
		<< sinc_mp
		<< std::endl;

	return 0;
}
